/* Copyright 2016 The TensorFlow Authors. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
==============================================================================*/
namespace vz_projector {

const BACKGROUND_COLOR = 0x7F7F7F;

const START_CAMERA_POS_3D = new THREE.Vector3(0, 0, 5.8);
const START_CAMERA_TARGET_3D = new THREE.Vector3(0, 0, 0);

const WIDTH = 250;
const HEIGHT = 250;

const CUBE_LENGTH = 2;
const MAX_ZOOM = 5 * CUBE_LENGTH;
const MIN_ZOOM = 0.025 * CUBE_LENGTH;

// Constants relating to the camera parameters.
const PERSP_CAMERA_FOV_VERTICAL = 70;
const PERSP_CAMERA_NEAR_CLIP_PLANE = 0.01;
const PERSP_CAMERA_FAR_CLIP_PLANE = 100;

// Key presses.
const SHIFT_KEY = 16;
const CTRL_KEY = 17;

const ORBIT_MOUSE_ROTATION_SPEED = 1;
const ORBIT_ANIMATION_ROTATION_CYCLE_IN_SECONDS = 7;

const BASIC_VERTEX_SHADER = [
  "varying vec3 vPosition;",
  "varying vec3 vNormal;",
  "void main() {",
  "vec4 Position = modelViewMatrix * vec4( position, 1.0 );",
  "vPosition = Position.xyz;",
  "vNormal = normalize( normalMatrix * normal );",
  "gl_Position = projectionMatrix * modelViewMatrix * vec4( position, 1.0 );",
  "}"
].join("\n");

const DIRECTION_VERTEX_SHADER = BASIC_VERTEX_SHADER;

const DIRECTION_FRAGMENT_SHADER = [
  "#include <common>",
  "#include <bsdfs>",
  "#include <lights_pars>",
  "varying vec3 vPosition;",
  "varying vec3 vNormal;",
  "void main() {",
  "vec3 normal = normalize( vNormal );",
  "GeometricContext geometry;",
  "geometry.normal = normal;",
  "geometry.position = vPosition;",
  "IncidentLight directLight;",
  "getPointDirectLightIrradiance(pointLights[0], geometry, directLight);",
];

const VIEW_DIRECTION_FRAGMENT_SHADER = DIRECTION_FRAGMENT_SHADER.concat([
  "gl_FragColor = vec4(normalize(-vPosition), 1.0);",
  "}"
]).join("\n");

const LIGHT_DIRECTION_FRAGMENT_SHADER = DIRECTION_FRAGMENT_SHADER.concat([
  "gl_FragColor = vec4(normalize(directLight.direction), 1.0);",
  "}"
]).join("\n");

const NORMAL_VERTEX_SHADER = BASIC_VERTEX_SHADER;

const NORMAL_FRAGMENT_SHADER = [
  "varying vec3 vPosition;",
  "varying vec3 vNormal;",
  "void main() {",
  "vec3 normal = normalize( vNormal );",
  "gl_FragColor = vec4(normal, 1.0);",
  "}"
].join("\n");

const INPUT_VERTEX_SHADER = BASIC_VERTEX_SHADER;

const INPUT_FRAGMENT_SHADER = [
  "const float PI = 3.1415926535897932384626433832795;",
  "const float PI_2 = 1.57079632679489661923;",
  "vec3 DirectionsToRusinkLocal(vec3 a, vec3 b, vec3 norm){",
  "vec3 H = normalize((a+b)/2.0);",
  "float thetaH = acos(clamp(dot(H, norm), -1.0, 1.0));",
  "float thetaD = acos(clamp(dot(H, a), -1.0, 1.0));",
  "vec3 w = dot(H,norm)*H-norm;",
  "float phiD = acos(clamp(dot(w, normalize(b-a)), -1.0, 1.0));",
  "return vec3(phiD, thetaH, thetaD);",
  "}"
].concat(DIRECTION_FRAGMENT_SHADER.concat([
  "vec3 xGrid = DirectionsToRusinkLocal(normalize(-vPosition), normalize(directLight.direction), normal);",
  "xGrid.x /= PI;",
  "xGrid.y /= PI_2;",
  "xGrid.z /= PI_2;",
  "float xGridAdd = xGrid.x*2.0*PI;",
  "xGrid.x = sin(xGridAdd)*0.5+0.5;",
  "xGridAdd = cos(xGridAdd)*0.5+0.5;",
  "gl_FragColor = vec4(xGridAdd, xGrid);",
  "}"
])).join("\n");

const interpShader = {
  uniforms: {
    "p0":{value:null},
    "p1":{value:null},
    "t":{value:0.5},
  },
  vertexShader: [
    "varying vec2 vUv;",
    "void main() {",
    "vUv = uv;",
    "gl_Position = projectionMatrix * modelViewMatrix * vec4( position, 1.0 );",
    "}"
  ].join("\n"),
  fragmentShader: [
    "uniform sampler2D p0;",
    "uniform sampler2D p1;",
    "uniform float t;",
    "varying vec2 vUv;",
    "void main() {",
    "vec4 c0 = texture2D( p0, vUv );",
    "vec4 c1 = texture2D( p1, vUv );",
    "gl_FragColor = mix(c0, c1, t);",
    "}"
  ].join("\n")
};

const stencilShader = {
  uniforms: {},
  vertexShader: [
    "void main() {",
    "gl_Position = projectionMatrix * modelViewMatrix * vec4( position, 1.0 );",
    "}"
  ].join("\n"),
  fragmentShader: [
    "void main() {",
    "gl_FragColor = vec4(1.0, 1.0, 1.0, 1.0);",
    "}"
  ].join("\n")
};

const LIGHTING_VERTEX_SHADER = BASIC_VERTEX_SHADER;

const LIGHTING_FRAGMENT_SHADER = [
  "#include <common>",
  "#include <bsdfs>",
  "#include <lights_pars>",
  "uniform sampler2D brdfTex;",
  "uniform int width;",
  "uniform int height;",
  "uniform float gamma;",
  "varying vec3 vPosition;",
  "varying vec3 vNormal;",
  "vec3 brdfMapping(vec3 brdf){",
  "  brdf = brdf*0.15;",
  "  for(int i=0;i<4;i++)",
  "    brdf = exp(brdf)-vec3(1.0, 1.0, 1.0);",
  "  return brdf;",
  "}",
  "void main() {",
  "vec3 normal = normalize( vNormal );",
  "GeometricContext geometry;",
  "geometry.normal = normal;",
  "geometry.position = vPosition;",
  "IncidentLight directLight;",
  "getPointDirectLightIrradiance(pointLights[0], geometry, directLight);",
  "float dotNL = saturate(dot(geometry.normal, directLight.direction));",
  "vec3 irradiance = dotNL * directLight.color;",
  "vec2 uv = gl_FragCoord.xy / vec2(width, height);",
  "vec3 brdf = texture2D(brdfTex, vec2(uv.x, 1.0 - uv.y)).rgb;",
  "brdf = max(brdfMapping(brdf), 0.0);",
  "vec3 reflectance = irradiance*brdf;",
  "reflectance = pow(reflectance, vec3(1.0/gamma));",
  "gl_FragColor = vec4(reflectance, 1.0);",
  "}"
].join("\n");

export class BrdfRenderer{
  // private projector: Projector;

  // private onCameraMoveListeners: OnCameraMoveListener[] = [];

  private height: number;
  private width: number;
  private gamma: number;

  // private mouseMode: MouseMode;
  // private backgroundColor: number = BACKGROUND_COLOR;

  // private dimensionality: number = 3;
  private renderer: THREE.WebGLRenderer;

  private scene: THREE.Scene;
  // private pickingTexture: THREE.WebGLRenderTarget;
  private pointLight: THREE.PointLight;
  // private directionalLight: THREE.DirectionalLight;

  private cameraDef: CameraDef = null;
  private camera: THREE.Camera;
  // private orbitAnimationOnNextCameraCreation: boolean = false;
  // private orbitCameraControls: any;
  // private orbitAnimationId: number;

  // private materials: THREE.ShaderMaterial[] = [];
  // private renderTargets: THREE.WebGLRenderTarget[] = [];
  // private renderTarget: THREE.WebGLRenderTarget;
  // private composer: THREE.EffectComposer;
  private stencilEffect: THREE.ShaderPass;
  private tensorMaterial: THREE.ShaderMaterial;
  // private warmUpMaterial: THREE.MeshBasicMaterial;
  private lightingMaterial: THREE.ShaderMaterial;

  private gui: dat.GUI;
  // private _camreaPosition: THREE.Vector3;

  private model: tf.LayersModel;
  private gvses: tf.Tensor[];
  private gvs: tf.Tensor;
  private brdfTexture: THREE.DataTexture;

  private profiler:{};

  private first: boolean;

  private tfBackend: tf.MathBackendWebGL;
  private tfGl: WebGL2RenderingContext;
  // private shaderProgram: WebGLProgram;
  // private aVertLocation: GLint;
  // private aTexLocation: GLint;
  // private vertex_buffer: WebGLBuffer;
  // private texture_buffer: WebGLBuffer;
  // private index_buffer: WebGLBuffer;
  // private tfColorTexture: WebGLTexture;
  // private tfImage: HTMLImageElement;

 
  constructor(
    private container: HTMLElement) {

    this.initTFGraphics();
    this.initGraphics();
    this.initGUI();
    this.initModel();

    // projector.registerSelectionChangedListener( (selection, neighbors) =>
    //   this.updateGVS(selection, neighbors));
    this.first = true;
  }
  
  private initGraphics(){
    this.scene = new THREE.Scene();
    
    // const canvas = document.createElement( 'canvas' );
    // const context = canvas.getContext( 'webgl2' ) as WebGLRenderingContext;
    // this.renderer = new THREE.WebGLRenderer(
    //   {canvas: canvas, context: context, alpha: true, premultipliedAlpha: false, antialias: false});
    // this.renderer = new THREE.WebGLRenderer(
    //     {canvas: this.tfGl.canvas, context: this.tfGl, alpha: true, premultipliedAlpha: false, antialias: false});
    this.renderer = new THREE.WebGLRenderer(
        {alpha: true, premultipliedAlpha: false, antialias: true});
    this.renderer.setClearColor(BACKGROUND_COLOR, 1);
    //
    this.width = WIDTH; 
    this.height = HEIGHT;
    this.gamma = 2.2;
    this.renderer.setSize(this.width, this.height);
    this.container.appendChild(this.renderer.domElement);
    
    this.pointLight = new THREE.PointLight(0xffffff, 1);
    this.pointLight.position.set(50, 50, 50);
    this.scene.add(this.pointLight);

    // this.directionalLight = new THREE.DirectionalLight(0xffffff, 0.5);
    // this.scene.add(this.directionalLight );

    // this.materials = [ 
      // new THREE.ShaderMaterial({
      //   uniforms: THREE.UniformsUtils.merge([
      //       THREE.UniformsLib.lights,
      //     ]),
      //   vertexShader: DIRECTION_VERTEX_SHADER,
      //   fragmentShader: VIEW_DIRECTION_FRAGMENT_SHADER,
      //   lights: true,
      //   }),
      // new THREE.ShaderMaterial({
      //   uniforms: THREE.UniformsUtils.merge([
      //       THREE.UniformsLib.lights,
      //     ]),
      //   vertexShader: DIRECTION_VERTEX_SHADER,
      //   fragmentShader: LIGHT_DIRECTION_FRAGMENT_SHADER,
      //   lights: true,
      //   }),
      // new THREE.ShaderMaterial({
      //   uniforms: {},
      //   vertexShader: NORMAL_VERTEX_SHADER,
      //   fragmentShader: NORMAL_FRAGMENT_SHADER,
      //   }),
    this.tensorMaterial = new THREE.ShaderMaterial({
        uniforms: THREE.UniformsUtils.merge([
            THREE.UniformsLib.lights,
        ]),
        vertexShader: INPUT_VERTEX_SHADER,
        fragmentShader: INPUT_FRAGMENT_SHADER,
        lights: true,
        }),  
    // ];

    this.lightingMaterial = new THREE.ShaderMaterial({
      uniforms: THREE.UniformsUtils.merge([
          THREE.UniformsLib.lights,
          {
            "brdfTex":{value:null},
            "width":{value:this.width},
            "height":{value:this.height},
            "gamma":{value:this.gamma},
          },
      ]),
      vertexShader: LIGHTING_VERTEX_SHADER,
      fragmentShader: LIGHTING_FRAGMENT_SHADER,
      lights: true,
    });

    {
      const geometry = new THREE.SphereBufferGeometry(3, 32, 32);
      const mesh = new THREE.Mesh(geometry, this.tensorMaterial);
      mesh.position.set(0, 0, 0);
      this.scene.add(mesh);
    }
    // {
    //   const geometry = new THREE.BoxGeometry(1, 1, 1);
    //   const mesh = new THREE.Mesh(geometry, this.materials[0]);
    //   mesh.position.set(0, 3, 0);
    //   this.scene.add(mesh);
    // }

    this.cameraDef = this.makeDefaultCameraDef();
    this.makeCamera3D(this.cameraDef, this.width, this.height);
    // this.orbitCameraControls.minDistance = MIN_ZOOM;
    // this.orbitCameraControls.maxDistance = MAX_ZOOM;
    // this.orbitCameraControls.update();

    // const renderCanvasSize = this.renderer.getSize();
    // const pixelRatio = this.renderer.getPixelRatio();
    // this.renderTargets = new Array<THREE.WebGLRenderTarget>(this.materials.length);
    // for (var i = 0; i < this.renderTargets.length; i++) {
    //   this.renderTargets[i] = new THREE.WebGLRenderTarget(renderCanvasSize.width * pixelRatio, renderCanvasSize.height * pixelRatio);
    //   this.renderTargets[i].texture.format = THREE.RGBFormat;
    //   this.renderTargets[i].texture.minFilter = THREE.NearestFilter;
    //   this.renderTargets[i].texture.magFilter = THREE.NearestFilter;
    //   this.renderTargets[i].texture.type = THREE.FloatType;
    //   this.renderTargets[i].texture.generateMipmaps = false;
    //   this.renderTargets[i].stencilBuffer = false;
    //   this.renderTargets[i].depthBuffer = true;
    // }

    this.stencilEffect = new THREE.ShaderPass(stencilShader);
    this.stencilEffect.renderToScreen = true;

    // const pixel = new Float32Array(3*this.width*this.height);
    // this.brdfTexture = new THREE.DataTexture(pixel, this.width, this.height, THREE.RGBFormat, 
    //   THREE.FloatType, THREE.UVMapping, THREE.ClampToEdgeWrapping, THREE.ClampToEdgeWrapping,
    //   THREE.NearestFilter, THREE.NearestFilter);
    // this.brdfTexture.needsUpdate = true;

    // const forceTextureInitialization = function() {
    //   const material = new THREE.MeshBasicMaterial();
    //   const geometry = new THREE.PlaneBufferGeometry(1, 1, 1);
    //   const scene = new THREE.Scene();
    //   scene.add(new THREE.Mesh(geometry, material));
    //   const camera = new THREE.Camera();
  
    //   return function forceTextureInitialization(renderer, texture) {
    //     material.map = texture;
    //     renderer.render(scene, camera);
    //   };
    // }();

    // forceTextureInitialization(this.renderer, this.brdfTexture);
  }

  private initTFGraphics(){
    // this.tfImage = document.createElement('img');
    // this.tfImage.crossOrigin = '';
    // this.tfImage.src = 'https://i.imgur.com/vT3AkB6.jpg';

    this.tfBackend = tf.backend() as tf.MathBackendWebGL;
    this.tfGl = this.tfBackend.getGPGPUContext().gl as WebGL2RenderingContext;
    // this.tfManager = new GLContextManager(this.tfGl);

    // this.container.appendChild(this.tfGl.canvas);

    // const gl = this.tfGl;
    // const currentFramebuffer = gl.getParameter(gl.FRAMEBUFFER_BINDING);
    // const currentTexture = gl.getParameter(gl.TEXTURE_BINDING_2D);

    // this.tfColorTexture = gl.createTexture();
    // gl.bindTexture(gl.TEXTURE_2D, this.tfColorTexture);
    // gl.texImage2D(gl.TEXTURE_2D, 0, gl.RGBA32F, this.width, this.height, 
    //                             0, gl.RGBA, gl.FLOAT, null);

    // this.tfFramebuffer = gl.createFramebuffer();
    // gl.bindFramebuffer(gl.FRAMEBUFFER, this.tfFramebuffer);
    // gl.framebufferTexture2D( gl.FRAMEBUFFER, gl.COLOR_ATTACHMENT0, 
    //   gl.TEXTURE_2D, this.tfColorTexture, 0 );

    // gl.bindFramebuffer( gl.FRAMEBUFFER, null );
    // this.GLCheck();
    // const colorBuffer = gl.createRenderbuffer();
    // gl.bindRenderbuffer(gl.RENDERBUFFER, colorBuffer);
    // gl.renderbufferStorage(gl.RENDERBUFFER, gl.RGBA4, this.width, this.height);
    // gl.framebufferRenderbuffer(gl.FRAMEBUFFER, gl.COLOR_ATTACHMENT0, 
    //   gl.RENDERBUFFER, colorBuffer);
    // this.GLCheck();

    //   // Vertex shader source code
    // var vertCode = `
    //   attribute vec2 coordinates;
    //   attribute vec2 aTexCoord;
    //   varying highp vec2 vTexCoord;
    //   void main(void) {
    //     gl_Position = vec4(coordinates,1.0,1.0);
    //     vTexCoord = aTexCoord;
    //   }
    // `;
    // var vertShader = gl.createShader(gl.VERTEX_SHADER);
    // gl.shaderSource(vertShader, vertCode);
    // gl.compileShader(vertShader);

    // //fragment shader source code
    // var fragCode = `
    //   precision mediump float;
    //   uniform sampler2D texture;
    //   varying highp vec2 vTexCoord;
    //   void main(void) {
    //     gl_FragColor = texture2D(texture, vTexCoord);
    //   }
    // `;
    // var fragShader = gl.createShader(gl.FRAGMENT_SHADER);
    // gl.shaderSource(fragShader, fragCode);
    // gl.compileShader(fragShader);

    // this.shaderProgram = gl.createProgram();
    // gl.attachShader(this.shaderProgram, vertShader);
    // gl.attachShader(this.shaderProgram, fragShader);
    // gl.deleteShader( vertShader );
    // gl.deleteShader( fragShader );
    // gl.linkProgram(this.shaderProgram);
    // gl.useProgram(this.shaderProgram);

    // this.aVertLocation = gl.getAttribLocation(this.shaderProgram, "coordinates");
    // this.aTexLocation = gl.getAttribLocation(this.shaderProgram, "aTexCoord");

    // const indices = [0, 1, 2, 0, 2, 3];
    // const vertices = [-1, -1, 1, -1, 1, 1, -1, 1];
    // const texCoords = [0, 0, 1, 0, 1, 1, 0, 1];

    // const current_array_buffer = gl.getParameter(gl.ARRAY_BUFFER_BINDING);
    // this.vertex_buffer = gl.createBuffer();
    // gl.bindBuffer(gl.ARRAY_BUFFER, this.vertex_buffer);
    // gl.enableVertexAttribArray(this.aVertLocation);
    // gl.vertexAttribPointer(this.aVertLocation, 2, gl.FLOAT, false, 0, 0);
    // gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(vertices), gl.STATIC_DRAW);
    // gl.bindBuffer(gl.ARRAY_BUFFER, null);

    // this.texture_buffer = gl.createBuffer();
    // gl.bindBuffer(gl.ARRAY_BUFFER, this.texture_buffer);
    // gl.enableVertexAttribArray(this.aTexLocation);
    // gl.vertexAttribPointer(this.aTexLocation, 2, gl.FLOAT, false, 0, 0);
    // gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(texCoords), gl.STATIC_DRAW);
    // gl.bindBuffer(gl.ARRAY_BUFFER, null);
    // gl.bindBuffer(gl.ARRAY_BUFFER, current_array_buffer);

    // const current_index_buffer = gl.getParameter(gl.ELEMENT_ARRAY_BUFFER_BINDING);
    // this.index_buffer = gl.createBuffer();
    // gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, this.index_buffer);
    // gl.bufferData(gl.ELEMENT_ARRAY_BUFFER, new Uint16Array(indices), gl.STATIC_DRAW);
    // gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, current_index_buffer);
    // this.GLCheck();

    // this.tfTexture = gl.createTexture();
    // gl.bindTexture(gl.TEXTURE_2D, this.tfTexture);
    // gl.uniform1i( gl.getUniformLocation( this.shaderProgram, 'texture' ), 0 );
    // gl.bindTexture(gl.TEXTURE_2D, null);

    // gl.bindFramebuffer(gl.FRAMEBUFFER, currentFramebuffer);
    // gl.bindTexture(gl.TEXTURE_2D, currentTexture);
  }

  private initGUI(){
    // this.gui = new dat.GUI();
    // const scene = this.gui.addFolder('Scene');
    // scene.add(this, "gamma", 1, 5).step(0.1);

    // const camera = scene.addFolder('Camera');

    // this._camreaPosition = new THREE.Vector3().copy(this.camera.position);
    // camera.add(this._camreaPosition, 'x').listen();
    // camera.add(this._camreaPosition, 'y').listen();
    // camera.add(this._camreaPosition, 'z').listen();
    // camera.add(this.camera.position, 'x').listen();
    // camera.add(this.camera.position, 'y').listen();
    // camera.add(this.camera.position, 'z').listen();

    // const pointLight = scene.addFolder('Point Light');
    // pointLight.add(this.pointLight.position, 'x');
    // pointLight.add(this.pointLight.position, 'y');
    // pointLight.add(this.pointLight.position, 'z');
    // pointLight.add(this.pointLight, 'intensity', 0, 10).step(0.2);
    // pointLight.add(this.pointLight, 'decay', 1, 3).step(0.1);
    // pointLight.add(this.pointLight, 'distance', 0, 10).step(0.2);

    // const profiler = this.gui.addFolder('Profiler/ms');
    this.profiler = {
      total:0.1,
      tensorPass:0.1,
      pixelToInputTensor:0.1,
      maskPass:0.1,
      pixelToMaskTensor:0.1,
      predict:0.1,
      brdfTensorToPixel:0.1,
      lightingPass:0.1
    }
    // Object.keys(this.profiler).map((x)=>{
    //   profiler.add(this.profiler, x).listen();
    // })
    // const directionalLight = folder.addFolder('Directional Light');
    // directionalLight.add(this.directionalLight.position, 'x');
    // directionalLight.add(this.directionalLight.position, 'y');
    // directionalLight.add(this.directionalLight.position, 'z');
    // directionalLight.add(this.directionalLight, 'intensity', 0, 10).step(0.2);
    // folder.add(this, 'width', 0, 1000).step(5).onChange((value)=>this.renderer.setSize(this.width, this.height));
    // folder.add(this, 'height', 0, 1000).step(5).onChange((value)=>this.renderer.setSize(this.width, this.height));
  }

  private initModel(){
    const loadModel = tf.loadLayersModel("models/model.json").then((model)=>{
      this.model = model;

      // Warmup the model before using real data.
      // const input = tf.zeros();
      // const gvs = tf.zeros([8]);
      // const mask = tf.zeros([1, this.width, this.height, 1]);
      // const warmupResult = model.predict([input, gvs, mask]) as tf.Tensor;
      // warmupResult.dataSync();
      // warmupResult.dispose();
    })

    const gvsData = tf.data.csv("models/STDv7RGB-gvs.csv");
    const loadGvs = gvsData.toArray().then((data)=>{
      const values = Object.keys(data).map((x)=>data[x]);
      const firstRow = Object.keys(values[0]).map((x)=>Number(x));
      const otherRows = values.map((x)=>Object.keys(x).map((key)=>x[key]));
      const arrays = [firstRow].concat(otherRows);
      this.gvses = tf.tensor(arrays).unstack();
    });

    Promise.all([loadModel, loadGvs]).then(()=>{
      // this.update();
    })
  }

  updateMaterial(index0: number, index1: number, t: number){
    if(this.gvses == undefined) return;
    if(index0 >=0 && index1 < 0){
      this.gvs = this.gvses[index0];
    }
    else if(index0 <0 && index1 >=0){
      this.gvs = this.gvses[index1];
    }
    else{
      const gvs0 = this.gvses[index0];
      const gvs1 = this.gvses[index1];
      this.gvs = gvs0.mul(1-t).add(gvs1.mul(t));
    }
    this.update();
  }

  private update(){
    // this._camreaPosition.copy(this.camera.position);
    this.profiler['total'] = performance.now();
    this.render();
    this.profiler['total']  = performance.now() - this.profiler['total'];
    // requestAnimationFrame(() => this.update());
  }

  // private getLayoutValues(): vector.Point2D {
  //   const width = this.container.offsetWidth;
  //   const height = Math.max(1, this.container.offsetHeight);
  //   return [width, height];
  // }

  private makeDefaultCameraDef(): CameraDef {
    const def = new CameraDef();
    def.zoom = 1.0;
    def.position =
        [START_CAMERA_POS_3D.x, START_CAMERA_POS_3D.y, START_CAMERA_POS_3D.z];
    def.target = [
      START_CAMERA_TARGET_3D.x, START_CAMERA_TARGET_3D.y,
      START_CAMERA_TARGET_3D.z
    ];
    return def;
  }

  // private makeOrbitControls(
  //     camera: THREE.Camera, cameraDef: CameraDef, cameraIs3D: boolean) {
  //   if (this.orbitCameraControls != null) {
  //     this.orbitCameraControls.dispose();
  //   }
  //   const occ =
  //       new (THREE as any).OrbitControls(camera, this.renderer.domElement);
  //   occ.target0 = new THREE.Vector3(
  //       cameraDef.target[0], cameraDef.target[1], cameraDef.target[2]);
  //   occ.position0 = new THREE.Vector3().copy(camera.position);
  //   occ.zoom0 = cameraDef.zoom;
  //   occ.enableRotate = cameraIs3D;
  //   occ.autoRotate = false;
  //   occ.rotateSpeed = ORBIT_MOUSE_ROTATION_SPEED;
  //   if (cameraIs3D) {
  //     occ.mouseButtons.ORBIT = THREE.MOUSE.LEFT;
  //     occ.mouseButtons.PAN = THREE.MOUSE.RIGHT;
  //   } else {
  //     occ.mouseButtons.ORBIT = null;
  //     occ.mouseButtons.PAN = THREE.MOUSE.LEFT;
  //   }
  //   occ.reset();

  //   this.camera = camera;
  //   this.orbitCameraControls = occ;
  // }

  private makeCamera3D(cameraDef: CameraDef, w: number, h: number) {
    let camera: THREE.PerspectiveCamera;
    {
      const aspectRatio = w / h;
      camera = new THREE.PerspectiveCamera(
          PERSP_CAMERA_FOV_VERTICAL, aspectRatio, PERSP_CAMERA_NEAR_CLIP_PLANE,
          PERSP_CAMERA_FAR_CLIP_PLANE);
      camera.position.set(
          cameraDef.position[0], cameraDef.position[1], cameraDef.position[2]);
      const at = new THREE.Vector3(
          cameraDef.target[0], cameraDef.target[1], cameraDef.target[2]);
      camera.lookAt(at);
      camera.zoom = cameraDef.zoom;
      camera.updateProjectionMatrix();
    }
    this.camera = camera;
    // this.makeOrbitControls(camera, cameraDef, true);
  }

  resize(size: vector.Point2D ) {
    // const [oldW, oldH] = [this.width, this.height];
    // const [newW, newH] = [this.container.offsetWidth, size[1]];
    // // const [newW, newH] = [oldW, oldH];

    // const camera = (this.camera as THREE.PerspectiveCamera);
    // camera.aspect = newW / newH;
    // camera.updateProjectionMatrix();

    // // Accouting for retina displays.
    // const dpr = window.devicePixelRatio || 1;
    // this.renderer.setPixelRatio(dpr);
    // this.renderer.setSize(newW, newH);

    // {
    //   const renderCanvasSize = this.renderer.getSize();
    //   const pixelRatio = this.renderer.getPixelRatio();
    //   for (var i = 0; i < this.renderTargets.length; i++)
    //     this.renderTargets[i].setSize(renderCanvasSize.width * pixelRatio, renderCanvasSize.height * pixelRatio);
    // }
  }

  private render() {
    if(this.gvs === undefined || this.model === undefined) return;

    const gl = this.renderer.context;
    
    this.GLCheck();
    // this.state.push();
    this.profiler['tensorPass'] = performance.now();
    // enable stencil test
    gl.enable( gl.STENCIL_TEST );
    gl.stencilFunc( gl.ALWAYS, 1, 0xff );
    gl.stencilOp( gl.REPLACE, gl.REPLACE, gl.REPLACE );
    this.renderer.autoClearStencil = true;
    this.GLCheck();
    this.scene.traverse((node)=>{
      if(node instanceof THREE.Mesh)
        node.material = this.tensorMaterial;
    })
    // if(this.first) console.log('tensor');
      this.GLCheck();
      // this.renderer.setRenderTarget(this.renderTarget);
      this.renderer.render(this.scene, this.camera);
      this.GLCheck();
    this.profiler['tensorPass'] = performance.now() - this.profiler['tensorPass'];
    // this.state.pop();

    this.profiler['pixelToInputTensor'] = performance.now();
      this.GLCheck();
      const input = tf.browser.fromPixels(this.renderer.domElement, 4).div(255).expandDims();
      this.GLCheck();
    this.profiler['pixelToInputTensor'] = performance.now() - this.profiler['pixelToInputTensor'];

    // change stencil function and operation for testing
    this.profiler['maskPass'] = performance.now();
      gl.stencilFunc( gl.EQUAL, 1, 0xff );
      gl.stencilOp( gl.KEEP, gl.KEEP, gl.KEEP );
      this.renderer.autoClearStencil = false;
      // if(this.first) console.log('stencil');
      this.GLCheck();
      this.stencilEffect.render(this.renderer, null, null, 0);
      this.GLCheck();
      gl.disable( gl.STENCIL_TEST );
    this.profiler['maskPass'] = performance.now() - this.profiler['maskPass'];

    //evaluate brdf
    this.profiler['pixelToMaskTensor'] = performance.now();
      this.GLCheck();
      const mask = tf.browser.fromPixels(this.renderer.domElement, 1).div(255).expandDims();
      this.GLCheck();
    this.profiler['pixelToMaskTensor'] = performance.now() - this.profiler['pixelToMaskTensor'];

    this.profiler['predict'] = performance.now();
      const gvs = this.gvs.expandDims();
      this.GLCheck();
      const output = this.model.predict([input, gvs, mask]) as tf.Tensor;
      this.GLCheck();
    this.profiler['predict'] = performance.now() - this.profiler['predict'];

    this.profiler['brdfTensorToPixel'] = performance.now();
      const data = output.dataSync();
      // const texData = output.dataResSync();
      // const data = this.test(texData);
      // const tfBrdfTexture = this.tfBackend.getTexture(output.dataId) as WebGLTexture;

      if(this.brdfTexture !== undefined)
        this.brdfTexture.dispose();
      this.brdfTexture = new THREE.DataTexture(data, this.width, this.height, THREE.RGBFormat, 
        THREE.FloatType, THREE.UVMapping, THREE.ClampToEdgeWrapping, THREE.ClampToEdgeWrapping,
        THREE.NearestFilter, THREE.NearestFilter);
      this.brdfTexture.needsUpdate = true;
    this.profiler['brdfTensorToPixel'] = performance.now() - this.profiler['brdfTensorToPixel'];

    this.profiler['lightingPass'] = performance.now();
      this.lightingMaterial.uniforms["brdfTex"].value = this.brdfTexture;
      this.lightingMaterial.uniforms["width"].value = this.width;
      this.lightingMaterial.uniforms["height"].value = this.height;
      this.lightingMaterial.uniforms["gamma"].value = this.gamma;
      this.scene.traverse((node)=>{
        if(node instanceof THREE.Mesh)
          node.material = this.lightingMaterial;
      });
      // if(this.first) console.log('lighting');
      this.renderer.render(this.scene, this.camera);
    this.profiler['lightingPass'] = performance.now() - this.profiler['lightingPass'];

    this.first = false;
  }

  // private drawBrdfTexture(){
  //   //draw brdf texture to framebuffer
  //   this.GLCheck();
  //   const currentFramebuffer = gl.getParameter(gl.FRAMEBUFFER_BINDING);
  //   const currentTexture = gl.getParameter(gl.TEXTURE_BINDING_2D);
  //   const currentProgram = gl.getParameter(gl.CURRENT_PROGRAM);   
  //   const currentElement = gl.getParameter(gl.ELEMENT_ARRAY_BUFFER_BINDING);   
    
  //   gl.bindFramebuffer(gl.FRAMEBUFFER, this.tfFramebuffer);
  //   gl.clearColor(0.0, 0.0, 0.0, 1.0);
  //   gl.clear( gl.COLOR_BUFFER_BIT | gl.DEPTH_BUFFER_BIT );
  //   gl.useProgram(this.shaderProgram);
  //   gl.bindTexture(gl.TEXTURE_2D, tfBrdfTexture);
    
  //   gl.enableVertexAttribArray(this.aVertLocation);
  //   gl.enableVertexAttribArray(this.aTexLocation);
  //     gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, this.index_buffer)
  //     gl.drawElements(gl.TRIANGLES, 6, gl.UNSIGNED_SHORT, 0);
  //   gl.disableVertexAttribArray(this.aVertLocation);
  //   gl.disableVertexAttribArray(this.aTexLocation);

  //   const packedRGBA = new Float32Array(this.width*this.height*4);
  //   gl.readPixels(0, 0, this.width, this.height, gl.RGBA, gl.FLOAT, packedRGBA);
  //   console.log(packedRGBA);
    
  //   gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, currentElement);
  //   gl.useProgram(currentProgram);
  //   gl.bindTexture(gl.TEXTURE_2D, currentTexture);
  //   gl.bindFramebuffer(gl.FRAMEBUFFER, currentFramebuffer); 
  // }
  private test(texData : tf.TextureData):Float32Array{
    const gl = this.tfGl;
    const {shape, dtype, texture, texShape} = texData;
    const batch = this.getBatchDim(shape);
    const [rows, cols] = this.getRowsCols(shape);
    const data = this.downloadMatrixFromPackedTexture(gl, texture, batch, rows, cols, texShape[0], texShape[1]);
    this.GLCheck();
    return data;
  }

  // private updateBrdfTexture(texData : tf.TextureData){
  //   const {shape, dtype, texture, texShape} = texData;
  //   const batch = this.getBatchDim(shape);
  //   const [rows, cols] = this.getRowsCols(shape);
  //   {
  //     const gl = this.tfGl;
  //     // const currentFramebuffer = gl.getParameter(gl.FRAMEBUFFER_BINDING);
  //     this.GLCheck();
  //     const tfBrdfTexture = this.tfBackend.getTexture(output.dataId) as WebGLTexture;
  //     //draw brdf texture to canvas in tf context
  //     this.GLCheck();
  //     gl.viewport(0, 0, this.width, this.height);
  //     this.GLCheck();
  //     // gl.bindFramebuffer(gl.FRAMEBUFFER, null);
  //     this.GLCheck();
  //     gl.clearColor(0.0, 0.0, 0.0, 1.0);
  //     gl.clear( gl.COLOR_BUFFER_BIT | gl.DEPTH_BUFFER_BIT );
  //     const currentTfProgram = gl.getParameter(gl.CURRENT_PROGRAM);   
  //     gl.useProgram(this.shaderProgram);
  //     const currentTfTexture = gl.getParameter(gl.TEXTURE_BINDING_2D);    
  //     gl.bindTexture(gl.TEXTURE_2D, tfBrdfTexture);
      
  //     gl.enableVertexAttribArray(this.aVertLocation);
  //     gl.enableVertexAttribArray(this.aTexLocation);
  //     gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, this.index_buffer)
  //     gl.drawElements(gl.TRIANGLES, 6, gl.UNSIGNED_SHORT, 0);
  //     gl.disableVertexAttribArray(this.aVertLocation);
  //     gl.disableVertexAttribArray(this.aTexLocation);
  //     gl.bindTexture(gl.TEXTURE_2D, currentTfTexture);
  //     gl.useProgram(currentTfProgram);
  //     this.GLCheck();
  //   }
  //   {
  //     //update brdf texture in our context from canvas in tf context
  //     const gl = this.renderer.context as WebGL2RenderingContext;
  //     const texProps = this.renderer.properties.get(this.brdfTexture);
  //     const brdfTexture = texProps.__webglTexture;
  //     this.GLCheck();
  //     const currentTexture = gl.getParameter(gl.TEXTURE_BINDING_2D);
  //     gl.bindTexture(gl.TEXTURE_2D, brdfTexture);
  //     this.GLCheck();
  //     gl.texImage2D(gl.TEXTURE_2D, 0, gl.RGBA32F, this.width, this.height, 0, gl.RGBA, gl.FLOAT, this.tfGl.canvas);
  //     gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MAG_FILTER, gl.LINEAR);
  //     gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MIN_FILTER, gl.LINEAR);
  //     gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_S, gl.CLAMP_TO_EDGE);
  //     gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_T, gl.CLAMP_TO_EDGE);
  //     gl.bindTexture(gl.TEXTURE_2D, currentTexture);
  //   }
  // }


  private getPackedMatrixTextureShapeWidthHeight(
    rows: number, columns: number): [number, number] {
  return [
    Math.max(1, Math.ceil(columns / 2)), Math.max(1, Math.ceil(rows / 2))
  ];
}

private getRowsCols(shape: number[]): [number, number] {
  if (shape.length === 0) {
    throw Error('Cannot get rows and columns of an empty shape array.');
  }

  return [
    shape.length > 1 ? shape[shape.length - 2] : 1, shape[shape.length - 1]
  ];
}
private sizeFromShape(shape: number[]): number {
  if (shape.length === 0) {
    // Scalar.
    return 1;
  }
  let size = shape[0];
  for (let i = 1; i < shape.length; i++) {
    size *= shape[i];
  }
  return size;
}

private getBatchDim(shape: number[], dimsToSkip = 2): number {
  return this.sizeFromShape(shape.slice(0, shape.length - dimsToSkip));
}

private bindTextureToFrameBuffer(gl: WebGLRenderingContext, texture: WebGLTexture, framebuffer: WebGLFramebuffer) {
  gl.bindFramebuffer(gl.FRAMEBUFFER, framebuffer);
  gl.framebufferTexture2D(
    gl.FRAMEBUFFER, gl.COLOR_ATTACHMENT0, gl.TEXTURE_2D, texture, 0);
  
  const status = gl.checkFramebufferStatus(gl.FRAMEBUFFER);
  if (status !== gl.FRAMEBUFFER_COMPLETE) {
    throw new Error(
        'Error binding framebuffer: ');
  }
}

private unbindTextureToFrameBuffer(gl: WebGLRenderingContext, framebuffer: WebGLFramebuffer) {
  gl.bindFramebuffer(gl.FRAMEBUFFER, framebuffer);
  gl.framebufferTexture2D(
    gl.FRAMEBUFFER, gl.COLOR_ATTACHMENT0, gl.TEXTURE_2D, null, 0);
}

private getPackedRGBAArraySizeFromMatrixShape(
  rows: number, columns: number): number {
  const [w, h] = this.getPackedMatrixTextureShapeWidthHeight(rows, columns);
  return w * h * 4;
}

private downloadMatrixFromPackedTexture( gl: WebGLRenderingContext, texture: WebGLTexture, batch: number, rows: number,
  cols: number, physicalRows: number, physicalCols: number): Float32Array {
    // console.log(texture, batch, rows, cols, physicalRows, physicalCols);
    const currentFramebuffer = gl.getParameter(gl.FRAMEBUFFER_BINDING);
    this.bindTextureToFrameBuffer(gl, texture, currentFramebuffer);
        const [w, h] = this.getPackedMatrixTextureShapeWidthHeight(physicalRows, physicalCols);
        const packedRGBA = new Float32Array(this.getPackedRGBAArraySizeFromMatrixShape(physicalRows, physicalCols));
        gl.readPixels(0, 0, w, h, gl.RGBA, gl.FLOAT, packedRGBA);
        const matrix = new Float32Array(batch*rows*cols);
        // const data = this.myDecode(
        //   packedRGBA, batch, rows, cols, matrix);
        // console.log(data);
        const tmatrix = new Float32Array(batch*rows*cols);
        const data = this.decodeMatrixFromPackedRGBA(
          packedRGBA, batch, rows, cols, tmatrix);
        // console.log(tdata);
    this.unbindTextureToFrameBuffer(gl, currentFramebuffer);
    return data;
} 

private UpdateTextureFromPackedTexture( gl: WebGLRenderingContext, texture: WebGLTexture, batch: number, rows: number,
  cols: number, physicalRows: number, physicalCols: number) {
    // console.log(texture, batch, rows, cols, physicalRows, physicalCols);
    const currentFramebuffer = gl.getParameter(gl.FRAMEBUFFER_BINDING);
    this.bindTextureToFrameBuffer(gl, texture, currentFramebuffer);
        const [w, h] = this.getPackedMatrixTextureShapeWidthHeight(physicalRows, physicalCols);
        // gl.readPixels(0, 0, w, h, gl.RGBA, gl.FLOAT, packedRGBA);

    this.unbindTextureToFrameBuffer(gl, currentFramebuffer);
} 

private UpdateTextureFromCanvas(){
  //update brdf texture in our context from canvas in tf context
  const gl = this.renderer.context as WebGL2RenderingContext;
  const texProps = this.renderer.properties.get(this.brdfTexture);
  const brdfTexture = texProps.__webglTexture;
  this.GLCheck();
  const currentTexture = gl.getParameter(gl.TEXTURE_BINDING_2D);
  gl.bindTexture(gl.TEXTURE_2D, brdfTexture);
  this.GLCheck();
  gl.texImage2D(gl.TEXTURE_2D, 0, gl.RGBA32F, this.width, this.height, 0, gl.RGBA, gl.FLOAT, this.tfGl.canvas);
  gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MAG_FILTER, gl.LINEAR);
  gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MIN_FILTER, gl.LINEAR);
  gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_S, gl.CLAMP_TO_EDGE);
  gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_T, gl.CLAMP_TO_EDGE);
  gl.bindTexture(gl.TEXTURE_2D, currentTexture);
}

private nearestLargerEven(val: number): number {
  return val % 2 === 0 ? val : val + 1;
}

private myDecode(
  packedRGBA: Float32Array, batches: number, rows: number, columns: number,
  matrix: Float32Array): Float32Array{
  const requiredSize = rows * columns;
    if (matrix.length < requiredSize) {
        throw new Error(
            `matrix length (${matrix.length}) must be >= ${requiredSize}`);
    }
    // let cnt = 0;
    for (let batch = 0; batch < batches; batch++) {
      for (let row = 0; row < rows; row++) {
        const physicalRow = batch*rows+row;
        const srcRow = Math.floor(physicalRow / 2);
        for (let column = 0; column < columns; column++) {
          const leftOverColumn = column%2;
          const leftOverRow = physicalRow%2;
          const leftOver = (leftOverRow*2+leftOverColumn);
          const srcColumn = Math.floor(column / 2);
          const srcOffset = (srcRow*Math.ceil(columns / 2) + srcColumn)*4 + leftOver;
          const dstOffset = batch*rows*columns + row*columns + column;
          matrix[dstOffset] = packedRGBA[srcOffset];
          // matrix[dstOffset] = srcOffset;
          // cnt++;
          // if(cnt<=4){
          //   console.log(dstOffset, srcOffset);
          //   console.log([srcRow, srcColumn, leftOver], [batch, row, column]);
          // }
        }
      }
    }
    return matrix;
}

private decodeMatrixFromPackedRGBA(
  packedRGBA: Float32Array, batches: number, rows: number, columns: number,
  matrix: Float32Array): Float32Array {
  const requiredSize = rows * columns;
  if (matrix.length < requiredSize) {
    throw new Error(
        `matrix length (${matrix.length}) must be >= ${requiredSize}`);
  }

  const oddWidth = (columns % 2) === 1;//3%2=1 -- true
  const oddHeight = (rows % 2) === 1;//100%2=0 -- false
  const widthInFullBlocks = Math.floor(columns / 2);//3/2=1
  const heightInFullBlocks = Math.floor(rows / 2);//100/2=50

  const texelsPerRow = Math.ceil(columns / 2);//2
  const texelsPerBatch = texelsPerRow * Math.ceil(rows / 2);//2*50=100

  const flattenedMatrixSize =
      this.nearestLargerEven(rows) * this.nearestLargerEven(columns);//100*4=400

  for (let batch = 0; batch < batches; batch++) {
    const batchOffset = batch * rows * columns;//100b*3=300b
    const sourceOffset = batch * flattenedMatrixSize;//b*400=400b

    // loop over full 2x2 blocks
    {
      const srcStride = oddWidth ? 4 : 0;//4
      const dstStride = columns + (oddWidth ? 1 : 0);//3+1=4
      let src = sourceOffset;//
      let dstRow1 = batchOffset;
      let dstRow2 = batchOffset + columns;
      for (let blockY = 0; blockY < heightInFullBlocks; ++blockY) {
        for (let blockX = 0; blockX < widthInFullBlocks; ++blockX) {
          matrix[dstRow1++] = packedRGBA[src++];
          matrix[dstRow1++] = packedRGBA[src++];
          matrix[dstRow2++] = packedRGBA[src++];
          matrix[dstRow2++] = packedRGBA[src++];
          // matrix[dstRow1++] = src++;
          // matrix[dstRow1++] = src++;
          // matrix[dstRow2++] = src++;
          // matrix[dstRow2++] = src++;
        }
        src += srcStride;
        dstRow1 += dstStride;
        dstRow2 += dstStride;
      }
    }

    // loop down final column
    if (oddWidth) {
      let src = sourceOffset + (texelsPerRow - 1) * 4;
      let dst = batchOffset + columns - 1;
      const srcStride = texelsPerRow * 4;
      const dstStride = 2 * columns;
      for (let blockY = 0; blockY < heightInFullBlocks; ++blockY) {
        matrix[dst] = packedRGBA[src];
        matrix[dst + columns] = packedRGBA[src + 2];
        // matrix[dst] = src;
        // matrix[dst + columns] = src + 2;
        src += srcStride;
        dst += dstStride;
      }
    }

    // loop across final row
    if (oddHeight) {
      let src = sourceOffset + (texelsPerBatch - texelsPerRow) * 4;
      let dst = batchOffset + (rows - 1) * columns;
      for (let blockX = 0; blockX < widthInFullBlocks; ++blockX) {
        matrix[dst++] = packedRGBA[src++];
        matrix[dst++] = packedRGBA[src++];
        // matrix[dst++] = src++;
        // matrix[dst++] = src++;
        src += 2;
      }

      // fill in bottom-right cell
      if (oddWidth) {
        matrix[batchOffset + (rows * columns) - 1] = packedRGBA[src];
        // matrix[batchOffset + (rows * columns) - 1] = src;
      }
    }
  }

    return matrix;
  }

  private GLCheck(){
    if(this.first === true){
      {
        const gl = this.renderer.context;
        let err = gl.getError();
        if(err !== gl.NO_ERROR)
          console.trace();
        while(err !== gl.NO_ERROR){
          console.log("gl: "+ err);
          err = gl.getError();
        }
      }
      {
        const gl = this.tfGl;
        if(gl===undefined) return;
        let err = gl.getError();
        if(err !== gl.NO_ERROR)
          console.trace();
        if(err !== gl.NO_ERROR){
          console.log("tfgl: "+ err);
          err = gl.getError();
        }
      }
    }
  }
}

}  // namespace vz_projector
